@@ -8,6 +8,9 @@ from obspy.geodetics import gps2dist_azimuth
 from seisloc.dd import loadDD
 from seisloc.geometry import in_rectangle,loc_by_width
 from math import floor,ceil
+from seisloc.statistics import sum_count_Mo,write_sum_count_Mo
+from seisloc.phase_convert import cata2fdsn
+import logging
 
 
 class Catalog():
@@ -30,18 +33,21 @@ class Catalog():
             print("successfully load catalog file: "+locfile)
             self.init()
         else:
-            print("No hypoDD data provided, an empty Catalog created.")
-            print("You can define self.dict[evid] = [lon,lat,dep,mag,UTCDateTime]}")
-            print("Then run: .init() to initiate the catalog.")
+            print("*** No hypoDD data provided, an empty Catalog created.")
+            print("*** You can define self.dict[evid] = [lon,lat,dep,mag,UTCDateTime]}")
+            print("*** Then run: .init() to initiate the catalog.")
             self.dict = {}
 
     def init(self):
         """
         Initiate the catalog
         """
+        print(">>> Initiate the catalog ... ")
         self.init_keys()
         self.init_locs()
         self.init_relative_seconds()
+        self.sort()
+        print(">>> Initiation complted! ")
 
     def init_keys(self):
         """
@@ -153,7 +159,7 @@ class Catalog():
               markersize=6,
               size_ratio=1,
               imp_mag=None,
-              ref_time = UTCDateTime(2019,3,1),
+              reftime = None,
               cmap = None,
               vmin=0,
               vmax=1,
@@ -162,7 +168,9 @@ class Catalog():
               alonlat=[104,29],
               blonlat=[105,30],
               section_width=0.05,
-              crop=False):
+              crop=False,
+              ax = None,
+              plt_show=True):
         """
         Map view plot of earthquakes,earthquake denoted default by black circle
         Parameters:
@@ -172,7 +180,7 @@ class Catalog():
         |    edgecolor: earthquake marker(circle) edgecolor
         |      imp_mag: important magnitude. Magnitude larger than this level will be 
         |               highlighted
-        |     ref_time: reference time in UTCDateTime used to constrain colormap, if
+        |     reftime: reference time in UTCDateTime used to constrain colormap, if
         |               no colormap provided, seismicity will plotted by default
         |         cmap: colormap, check 'matplotlib' for more detail.
         |    vmin,vmax: the minimum and maximum value for colormap
@@ -184,16 +192,20 @@ class Catalog():
         |         crop: if True, the dataset will cut dataset to leave only events
                         inside the cross-section
         """
-        if figsize != None:
-            plt.figure(figsize=figsize)
-
+        #---------------- initiation ----------------------------------
+        if ax != None:     # get current axis
+            plt.sca(ax)
+        else:
+            if figsize != None:
+                plt.figure(figsize=figsize)
+        if reftime == None:
+            reftime = self.first_time + np.min(self.relative_seconds)
         if section_width <=0:
             raise Error("Width <= 0")
         # plot all events
         if add_section==True:
             alon = alonlat[0]; alat = alonlat[1]
             blon = blonlat[0]; blat = blonlat[1]
-            print(alon,alat,blon,blat,section_width)
             results = in_rectangle(self.locs,alon,alat,blon,blat,section_width/2)
             jj = np.where(results[:,0]==1)
             if crop == True:
@@ -211,7 +223,7 @@ class Catalog():
                     marker='o',
                     alpha=1)
         else:
-            shift_seconds = ref_time - self.first_time
+            shift_seconds = reftime - self.first_time
             times_plot = self.relative_seconds-shift_seconds
             if unit=="day":
                 times_plot = times_plot/(24*60*60)
@@ -219,7 +231,7 @@ class Catalog():
                 times_plot = times_plot/(60*60)
             elif unit=="minute":
                 times_plot = times_plot/60
-            plt.scatter(self.locs[:,0],
+            im = plt.scatter(self.locs[:,0],
                     self.locs[:,1],
                     c=times_plot,
                     s=(self.locs[:,3]+2)*size_ratio,
@@ -228,6 +240,8 @@ class Catalog():
                     vmax = vmax,
                     marker='o',
                     alpha=1)
+            cb = plt.colorbar(im)
+            cb.set_label(unit)
 
         # plot large events
         if imp_mag != None:
@@ -268,6 +282,9 @@ class Catalog():
         plt.xlabel("Longitude")
         plt.ylabel("Latitude")
         plt.gca().set_aspect("equal")
+
+        if plt_show == True:
+            plt.show()
         
     def vplot(self,
               alonlat,
@@ -278,12 +295,13 @@ class Catalog():
               depmax=10,
               size_ratio=1,
               imp_mag=None,
-              ref_time = UTCDateTime(2019,3,1),
+              reftime = UTCDateTime(2019,3,1),
               cmap=None,
               vmin=0,
               vmax=1,
               unit="day",
-              aspect="auto"):
+              aspect="auto",
+              plt_show=True):
         """
         Description
 
@@ -297,7 +315,7 @@ class Catalog():
         | edgecolor: earthquake marker(circle) edgecolor
         |   imp_mag: important magnitude. Magnitude larger than this level will be 
         |            highlighted
-        |  ref_time: reference time in UTCDateTime used to constrain colormap, if
+        |  reftime: reference time in UTCDateTime used to constrain colormap, if
         |            no colormap provided, seismicity will plotted by default
         |      cmap: colormap, check 'matplotlib' for more detail.
         | vmin,vmax: the minimum and maximum value for colormap
@@ -323,7 +341,7 @@ class Catalog():
                     facecolors='none',
                     s=(self.locs[jj,3]+2)*size_ratio*5)
         else:
-            shift_seconds = ref_time - self.first_time
+            shift_seconds = reftime - self.first_time
             times_plot = self.relative_seconds[jj]-shift_seconds
             if unit=="day":
                 times_plot = times_plot/(24*60*60)
@@ -362,19 +380,24 @@ class Catalog():
         plt.xlabel("distance (km)")
         plt.ylabel("depth (km)")
         plt.gca().set_aspect(aspect)
+
+        if plt_show==True:
+            plt.show()
     
     def MTplot(self,
                 xlim=[],
                 ylim=[0,5],
                 unit="day",
-                ref_time=UTCDateTime(2019,3,1),
+                reftime=None,
                 cmap=None,
                 vmin=0,
                 vmax=1,
-                figsize=(10,5)):
+                figsize=(10,5),
+                plt_show=True):
         """
         unit: 'day','hour' or 'second'
         """
+        # ----------------- initiate -----------------------------------
         fig,ax = plt.subplots(1,figsize=figsize)
         if unit == "day":
             denominator = (24*60*60)
@@ -385,25 +408,45 @@ class Catalog():
         elif unit == "second":
             denominator = 1
             plt.xlabel("Time (second)")
-        
+        if reftime == None:
+            reftime = self.first_time + np.min(self.relative_seconds)
+        print("Reference time is: ", reftime)
+        if len(xlim) == 0:
+            tbtime = self.first_time + np.min(self.relative_seconds)
+            tetime = self.first_time + np.max(self.relative_seconds)
+        if len(xlim) == 1:
+            tbtime = reftime+xlim[0]*denominator
+            tetime = self.first_time + np.max(self.relative_seconds)
+        if len(xlim) == 2:
+            tbtime = reftime+xlim[0]*denominator
+            tetime = reftime+xlim[1]*denominator
+
+        #----------------- processing ---------------------------------
         for key in self.keys:
             etime = self.dict[key][4]
+            if etime< tbtime or etime>tetime:
+                continue
             emag = self.dict[key][3]
-            diff_seconds = etime - ref_time
+            diff_seconds = etime - reftime
             diff_x = diff_seconds/denominator
             if cmap == None:
                 plt.plot([diff_x,diff_x],[ylim[0],emag],c='grey')
             else:
-                plt.plot([diff_x,diff_x],[mlow,emag],color=cmap((diff_x-vmin)/(vmax-vmin)))
+                plt.plot([diff_x,diff_x],[ylim[0],emag],color=cmap((diff_x-vmin)/(vmax-vmin)))
             plt.plot([diff_x],emag,'x',c='k')
         plt.ylim(ylim)
         if len(xlim)>0:
             plt.xlim(xlim)
+        else:
+            plt.xlim(left = (tbtime-reftime)/denominator-0.1,right = (tetime-reftime)/denominator+0.1)
         plt.ylabel("Magnitude")
 
+        if plt_show == True:
+            plt.show()
+
     def dep_dist_plot(self,refid=None,
                   refloc = [],
-                  ref_time=UTCDateTime(2019,3,1),
+                  reftime=None,
                   xlim=[],
                   deplim=[100,-4],
                   distlim=[],
@@ -411,7 +454,9 @@ class Catalog():
                   cmap=None,
                   vmin=0,
                   vmax=1,
-                  figsize=(8,6)):
+                  figsize=(8,6),
+                  plt_show=True):
+        #------------- initiation --------------------
         fig,axs = plt.subplots(2,1,figsize=figsize)
         if unit == "day":
             denominator = (24*60*60)
@@ -438,13 +483,16 @@ class Catalog():
             refdep = self.dict[refid][2]
         if len(refloc)>0:
             reflon,reflat,refdep = refloc
+        if reftime == None:
+            reftime = self.first_time + np.min(self.relative_seconds)
+
         for evid in self.keys:
             etime =self.dict[evid][4]
             elon = self.dict[evid][0]
             elat = self.dict[evid][1]
             edep = self.dict[evid][2]
             emag = self.dict[evid][3]
-            diff_x = (etime-ref_time)/denominator
+            diff_x = (etime-reftime)/denominator
             dist,_,_ = gps2dist_azimuth(elat,elon,reflat,reflon)
             d3dist = np.sqrt((dist/1000)**2+(edep-refdep)**2)
             if cmap==None:
@@ -453,8 +501,10 @@ class Catalog():
             else:
                 axs[0].scatter(diff_x,edep,s=(emag+2)*5,marker='o',color=cmap((diff_x-vmin)/(vmax-vmin)))
                 axs[1].scatter(diff_x,d3dist,s=(emag+2)*5,marker='o',color=cmap((diff_x-vmin)/(vmax-vmin)))
-
+    
         plt.tight_layout()
+        if plt_show == True:
+            plt.show()
         
     def depth_hist(self,mag_threshold=-9,depthmin=0,depthmax=10,gap=0.5):
         bins=np.arange(depthmin,depthmax,gap)
@@ -467,19 +517,27 @@ class Catalog():
         hist,bins = np.histogram(self.locs[:,2],bins=bins)
         ax.barh(bins[:-1]+gap/2,hist,height=gap,color='gray',edgecolor='k')
         ax.set_ylim([depthmax,depthmin])
-        plt.show()
+        return ax
         
-    def day_hist(self,ref_time=UTCDateTime(2019,1,1,0,0,0),xlim=[],ylim=[],color='b',edgecolor='k',plot_months=True,figsize=None):
+    def day_hist(self,
+                 reftime=UTCDateTime(2019,1,1,0,0,0),
+                 xlim=[],
+                 ylim=[],
+                 color='b',
+                 edgecolor='k',
+                 plot_months=True,
+                 figsize=None,
+                 plt_show = True):
         """
         Plot events by day-quantity in a histogram plot.
         Parameters:
-            -ref_time: Reference time for plot
+            -reftime: Reference time for plot
         """
         ref_list = []
         time_list = []
         for key in self.dict.keys():
             _,_,_,_,etime = self.dict[key]
-            ref_list.append((etime-ref_time)/(24*60*60))
+            ref_list.append((etime-reftime)/(24*60*60))
 
         min_day=floor(min(ref_list))
         max_day=ceil(max(ref_list))
@@ -494,9 +552,9 @@ class Catalog():
         # The top x-axis marks year and month in YYYYMM
         tick_list_1 = [] # Store the position number
         tick_list_2 = [] # Store the tick text
-        ref_year = ref_time.year
-        ref_month = ref_time.month
-        ref_day = ref_time.day
+        ref_year = reftime.year
+        ref_month = reftime.month
+        ref_day = reftime.day
         if ref_day == 1:
             tick_list_1.append(0)
             tick_list_2.append(str(ref_year)+str(ref_month).zfill(2))
@@ -508,7 +566,7 @@ class Catalog():
             tmp_year = loop_time.year
             tmp_month = loop_time.month
             loop_time = UTCDateTime(tmp_year,tmp_month,1)
-            diff_days = (loop_time - ref_time)/(24*60*60)
+            diff_days = (loop_time - reftime)/(24*60*60)
             if diff_days > (max_day):
                 status=False
             else:
@@ -529,7 +587,10 @@ class Catalog():
         ax1.set_xlabel("Time, days")
         ax1.set_ylabel("event quantity")
 
-    def diffusion_plot(self,refid=None,refloc=[],diff_cfs=[],unit="day",xlim=[],ylim=[]):
+        if plt_show == True:
+            plt_show()
+
+    def diffusion_plot(self,refid=None,refloc=[],diff_cfs=[],unit="day",xlim=[],ylim=[],plt_show=True):
         '''
         Parameters:
         refid: reference event id
@@ -588,6 +649,8 @@ class Catalog():
             plt.ylim(ylim)
         else:
             plt.ylim(bottom=0)
+        if plt_show == True:
+            plt_show()
 
     def animation(self,
                   incre_hour=2,
@@ -615,13 +678,14 @@ class Catalog():
             ylim = [np.min(self.locs[:,1]),np.max(self.locs[:,1])]    
         min_time = self.first_time+np.min(self.relative_seconds)
         max_time = self.first_time+np.max(self.relative_seconds)
+        
         if mb_time == None:
             mb_time = min_time
         if me_time == None:
             me_time = max_time
         print("Movie start time is: ",mb_time)
         print("  Movie end time is: ",me_time)
-
+        #---------- cmap -------------------------
         if vmin == None:
             vmin = 0
         if vmax == None:
@@ -630,7 +694,7 @@ class Catalog():
         inc_second = incre_hour*60*60 # Time increment
         loop_time = mb_time
         count = 1
-        ref_time = mb_time
+        reftime = mb_time
         while loop_time <= me_time:
             fig = plt.figure(1,figsize=(8,8))
             ax1 = fig.add_subplot(111)
@@ -668,13 +732,13 @@ class Catalog():
                 e_time = self.first_time + second
                 if e_time<(loop_time+inc_second/2) and e_time>mb_time:
                     eve_arr.append(self.locs[i,:4])
-                    rela_days.append((e_time-ref_time)/(24*60*60))
+                    rela_days.append((e_time-reftime)/(24*60*60))
             eve_arr = np.array(eve_arr)
             rela_days = np.array(rela_days)
 
             if len(eve_arr)>0:
                 if cmap != None:
-                        s_eve=ax1.scatter(eve_arr[:,0],
+                    s_eve=ax1.scatter(eve_arr[:,0],
                                           eve_arr[:,1],
                                           s=(eve_arr[:,3]+2)*5,
                                           c=rela_days,
@@ -683,7 +747,7 @@ class Catalog():
                                           vmax=vmax,
                                           label="Events")
                 else:
-                        s_eve=ax1.scatter(eve_arr[:,0],
+                    s_eve=ax1.scatter(eve_arr[:,0],
                                           eve_arr[:,1],
                                           s=(eve_arr[:,3]+2)*5,
                                           c='k',
@@ -717,7 +781,7 @@ class Catalog():
 
         axs = intervals_plot(xys=self.locs[:,:2],
                 rela_secs=self.relative_seconds,
-                ref_time=self.first_time,
+                reftime=self.first_time,
                 interval=interval,method=method,
                 xlim=xlim,ylim=ylim,
                 columns=columns,subplotsize=subplotsize,
@@ -742,6 +806,42 @@ class Catalog():
                           zorder=zorder,
                           wspace=None,hspace=None)
         return axs
+    def sum_count_Mo(self,starttime,endtime):
+        self.dict_count_Mo = sum_count_Mo(self,starttime,endtime)
+    def write_sum_count_Mo(self,outfile="sum_count_Mo.txt",mode='day'):
+        write_sum_count_Mo(self.dict_count_Mo,outfile,mode)
+
+    def write_info(self,info_file="lon_lat_dep_mag_relative_days.txt",reftime=None,disp=False):
+        if reftime == None:
+            reftime = self.first_time
+        print("The reference time is: ",reftime)
+        f = open(info_file,'w')
+        for key in self.keys:
+            lon = self.dict[key][0]
+            lat = self.dict[key][1]
+            dep = self.dict[key][2]
+            mag = self.dict[key][3]
+            relative_days = (self.dict[key][4]-reftime)/(24*60*60)
+            _key = format(key,'8d')
+            _lon = format(lon,'11.5f')
+            _lat = format(lat,'10.5f')
+            _dep = format(dep,'7.2f')
+            _mag = format(mag,'4.1f')
+            _relative_days = format(relative_days,'9.3f')
+            line = _key+_lon+_lat+_dep+_mag+_relative_days
+            f.write(line+"\n")
+            if disp == True:
+                print(line)
+        print("Catalog information write into: ",info_file)
+        f.close()
+
+    def cata2fdsn(self,author="Hardy",catalog="SC",
+                  cont="SC",contID="01",magtype="ML",
+                  magauthor="SC Agency",elocname="SC",out_file='cata.fdsn'):
+
+        cata2fdsn(self,author=author,catalog=catalog,                         
+                  cont=cont,contID=contID,magtype=magtype,                       
+                  magauthor=magauthor,elocname=elocname,out_file=out_file)
 
     def copy(self):
         return copy.deepcopy(self)
@@ -763,3 +863,18 @@ class Catalog():
     
     def __getitem__(self,key):
         return self.dict[key]
+
+def hypoinv2Catalog(inv):
+    """
+    Convert Hypoinv class to Catalog class
+    """
+    inv_dict={}
+    for key in inv.dict_evid.keys():
+        inv_dict[key] = inv.dict_evid[key][1:5]
+        _time = inv.dict_evid[key][0]
+        etime = UTCDateTime.strptime(_time,"%Y%m%d%H%M%S%f")
+        inv_dict[key].append(etime)
+    inv_cata = Catalog(locfile=None)
+    inv_cata.dict = inv_dict
+    inv_cata.init()
+    return inv_cata
